@@ -16,47 +16,49 @@
 #include "level.h"
 #include "path.h"
 #include "bottom.h"
-#include "index.h"
+#include "frontier.h"
 #include "callback.h"
-#include "bv.h"
+#include "rankedframe.h"
 #include "runset.h"
-#include "samplepred.h"
-#include "splitnode.h"
+#include "obspart.h"
+#include "splitfrontier.h"
 
 
 unsigned int Level::predFixed = 0;
 vector<double> Level::predProb;
 
-Level::Level(unsigned int _nSplit,
+Level::Level(IndexType nSplit_,
              unsigned int _nPred,
-             const vector<unsigned int> &_denseIdx,
-             unsigned int _nPredDense,
-             unsigned int bagCount,
-             unsigned int _idxLive,
+             const RankedFrame* rankedFrame,
+             IndexType bagCount,
+             IndexType idxLive_,
              bool _nodeRel,
              Bottom *_bottom) :
   nPred(_nPred),
-  denseIdx(_denseIdx),
-  nPredDense(_nPredDense),
-  nSplit(_nSplit),
+  denseIdx(rankedFrame->getDenseIdx()),
+  nPredDense(rankedFrame->getNPredDense()),
+  nSplit(nSplit_),
   noIndex(bagCount),
-  idxLive(_idxLive),
+  idxLive(idxLive_),
   defCount(0), del(0),
-  indexAnc(vector<IndexAnc>(nSplit)),
+  indexAnc(vector<IndexRange>(nSplit)),
   def(vector<MRRA>(nSplit * nPred)),
   denseCoord(vector<DenseCoord>(nSplit * nPredDense)),
-  relPath(new IdxPath(idxLive)),
-  offCand(vector<unsigned int>(nSplit * nPred)),
+  relPath(make_unique<IdxPath>(idxLive)),
+  offCand(vector<IndexType>(nSplit * nPred)),
   nodeRel(_nodeRel),
   bottom(_bottom)
 {
   MRRA df;
-  df.init();
+
   // Coprocessor only.
   fill(def.begin(), def.end(), df);
   fill(offCand.begin(), offCand.end(), bagCount);
 }
 
+Level::~Level() {
+}
+
 void Level::immutables(unsigned int feFixed, const vector<double> &feProb) {
   predFixed = feFixed;
   for (auto prob : feProb) {
@@ -76,7 +78,7 @@ bool Level::nonreachPurge() {
   for (unsigned int mrraIdx = 0; mrraIdx < nSplit; mrraIdx++) {
     if (liveCount[mrraIdx] == 0) {
       for (unsigned int predIdx = 0; predIdx < nPred; predIdx++) {
-        undefine(mrraIdx, predIdx); // Harmless if already undefined.
+        undefine(SplitCoord(mrraIdx, predIdx)); // Harmless if already undefined.
         purged = true;
       }
     }
@@ -89,66 +91,85 @@ bool Level::nonreachPurge() {
 void Level::flush(bool forward) {
   for (unsigned int mrraIdx = 0; mrraIdx < nSplit; mrraIdx++) {
     for (unsigned int predIdx = 0; predIdx < nPred; predIdx++) {
-      if (!isDefined(mrraIdx, predIdx))
+      SplitCoord splitCoord(mrraIdx, predIdx);
+      if (!isDefined(splitCoord))
         continue;
       if (forward) {
-        flushDef(mrraIdx, predIdx);
+        flushDef(splitCoord);
       }
       else {
-        undefine(mrraIdx, predIdx);
+        undefine(splitCoord);
       }
     }
   }
 }
 
 
-void Level::flushDef(unsigned int mrraIdx, unsigned int predIdx) {
+void Level::flushDef(const SplitCoord& splitCoord) {
   if (del == 0) // Already flushed to front level.
     return;
 
   unsigned int bufIdx;
   bool singleton;
-  consume(mrraIdx, predIdx, bufIdx, singleton);
-  frontDef(mrraIdx, predIdx, bufIdx, singleton);
+  consume(splitCoord, bufIdx, singleton);
+  frontDef(splitCoord, bufIdx, singleton);
   if (!singleton)
-    bottom->scheduleRestage(del, mrraIdx, predIdx, bufIdx);
+    bottom->scheduleRestage(del, splitCoord, bufIdx);
 }
 
 
-void Level::frontDef(unsigned int mrraIdx, unsigned int predIdx, unsigned int bufIdx, bool singleton) {
-  unsigned int pathStart = backScale(mrraIdx);
+void Level::frontDef(const SplitCoord& splitCoord, unsigned int bufIdx, bool singleton) {
+  unsigned int pathStart = splitCoord.backScale(del);
   for (unsigned int path = 0; path < backScale(1); path++) {
-    bottom->addDef(nodePath[pathStart + path].Idx(), predIdx, 1 - bufIdx, singleton);
+    bottom->addDef(SplitCoord(nodePath[pathStart + path].getSplitIdx(), splitCoord.predIdx), 1 - bufIdx, singleton);
   }
 }
 
 
-void Level::getBounds(const SPPair &mrra, unsigned int &startIdx, unsigned int &extent) {
-  indexAnc[mrra.first].Ref(startIdx, extent);
-  (void) adjustDense(mrra.first, mrra.second, startIdx, extent);
+IndexRange Level::getRange(const SplitCoord &mrra) {
+  IndexRange idxRange = indexAnc[mrra.nodeIdx];
+  adjustRange(mrra, idxRange);
+  return idxRange;
 }
 
 
-Level::~Level() {
-  delete relPath;
+IndexRange Level::adjustRange(const SplitCoord& splitCoord,
+                              const Frontier* frontier,
+                              unsigned int& implicit) const {
+  IndexSet iSet(frontier->getISet(splitCoord));
+  IndexRange idxRange;
+  idxRange.set(iSet.getStart(), iSet.getExtent());
+  implicit = isDense(splitCoord) ? denseCoord[denseOffset(splitCoord)].adjustRange(idxRange) : 0;
+
+  return idxRange;
 }
 
 
-void Level::offsetClone(const SPPair &mrra, unsigned int reachOffset[], unsigned int reachBase[]) {
-  unsigned int nodeStart = backScale(mrra.first);
+
+void Level::adjustRange(const SplitCoord& splitCoord,
+                        IndexRange& idxRange) const {
+  if (isDense(splitCoord)) {
+    (void) denseCoord[denseOffset(splitCoord)].adjustRange(idxRange);
+  }
+}
+
+
+
+void Level::offsetClone(const SplitCoord &mrra, unsigned int reachOffset[], unsigned int reachBase[]) {
+  unsigned int nodeStart = mrra.backScale(del);
   for (unsigned int i = 0; i < backScale(1); i++) {
-    reachOffset[i] = nodePath[nodeStart + i].IdxStart();
+    reachOffset[i] = nodePath[nodeStart + i].getIdxStart();
   }
   if (reachBase != nullptr) {
     for (unsigned int i = 0; i < backScale(1); i++) {
-      reachBase[i] = nodePath[nodeStart + i].RelBase();
+      reachBase[i] = nodePath[nodeStart + i].getRelBase();
     }
   }
 }
 
 
-void Level::setSingleton(unsigned int levelIdx, unsigned int predIdx) {
-  def[pairOffset(levelIdx, predIdx)].setSingleton();
+void Level::setSingleton(const SplitCoord& splitCoord) {
+  def[splitCoord.strideOffset(nPred)].setSingleton();
 }
 
 
@@ -167,7 +188,9 @@ void Level::reachingPaths() {
   vector<unsigned int> live(nSplit);
   vector<NodePath> path(backScale(nSplit));
   NodePath np;
-  np.init(noIndex, 0, 0, 0);
+  IndexRange bufRange;
+  bufRange.set(0,0);
+  np.init(noIndex, bufRange, 0);
   fill(path.begin(), path.end(), np);
   fill(live.begin(), live.end(), 0);
   
@@ -181,11 +204,11 @@ void Level::setExtinct(unsigned int idx) {
 }
 
 
-void Level::pathInit(const Bottom *bottom, unsigned int splitIdx, unsigned int path, unsigned int start, unsigned int extent, unsigned int relBase) {
+void Level::pathInit(const Bottom *bottom, unsigned int splitIdx, unsigned int path, const IndexRange& bufRange, unsigned int relBase) {
   unsigned int mrraIdx = bottom->getHistory(this, splitIdx);
   unsigned int pathOff = backScale(mrraIdx);
   unsigned int pathBits = path & pathMask();
-  nodePath[pathOff + pathBits].init(splitIdx, start, extent, relBase);
+  nodePath[pathOff + pathBits].init(splitIdx, bufRange, relBase);
   liveCount[mrraIdx]++;
 }
 
@@ -195,55 +218,53 @@ void Level::setLive(unsigned int idx, unsigned int path, unsigned int targIdx, u
 }
 
 
-bool Level::scheduleSplit(unsigned int splitIdx, unsigned int predIdx, unsigned int &rCount) const {
-  rCount = bottom->getRunCount(splitIdx, predIdx);
-  return !isSingleton(splitIdx, predIdx);
+bool Level::scheduleSplit(const SplitCoord& splitCoord, unsigned int &rCount) const {
+  rCount = bottom->getRunCount(splitCoord);
+  return !isSingleton(splitCoord);
 }
 
 
-// TODO:  Preempt overflow by walking wide subtrees depth-first.
-void Level::candidates(const IndexLevel *index, SplitNode *splitNode) {
-  int cellCount = nSplit * nPred;
+// TODO:  Preempt overflow by walking wide subtrees depth-nodeIdx.
+void Level::candidates(const Frontier* frontier, SplitFrontier *splitFrontier) {
+  IndexType cellCount = nSplit * nPred;
 
   auto ruPred = CallBack::rUnif(cellCount);
 
   vector<BHPair> heap(predFixed == 0 ? 0 : cellCount);
 
   unsigned int spanCand = 0;
-  for (unsigned int splitIdx = 0; splitIdx < nSplit; splitIdx++) {
-    unsigned int splitOff = splitIdx * nPred;
-    if (index->isUnsplitable(splitIdx)) { // Node cannot split.
+  for (IndexType splitIdx = 0; splitIdx < nSplit; splitIdx++) {
+    IndexType splitOff = splitIdx * nPred;
+    if (frontier->isUnsplitable(splitIdx)) { // Node cannot split.
       continue;
     }
     else if (predFixed == 0) { // Probability of predictor splitable.
-      candidateProb(splitNode, splitIdx, &ruPred[splitOff], index->getExtent(splitIdx), spanCand);
+      candidateProb(splitFrontier, splitIdx, &ruPred[splitOff], spanCand);
     }
     else { // Fixed number of predictors splitable.
-      candidateFixed(splitNode, splitIdx, &ruPred[splitOff], &heap[splitOff], index->getExtent(splitIdx), spanCand);
+      candidateFixed(splitFrontier, splitIdx, &ruPred[splitOff], &heap[splitOff], spanCand);
     }
   }
   setSpan(spanCand);
 }
 
 
-void Level::candidateProb(SplitNode *splitNode,
+void Level::candidateProb(SplitFrontier *splitFrontier,
                           unsigned int splitIdx,
                           const double ruPred[],
-                          unsigned int extent,
                           unsigned int &spanCand) {
   for (unsigned int predIdx = 0; predIdx < nPred; predIdx++) {
     if (ruPred[predIdx] < predProb[predIdx]) {
-      (void) preschedule(splitNode, splitIdx, predIdx, extent, spanCand);
+      (void) preschedule(splitFrontier, SplitCoord(splitIdx, predIdx), spanCand);
     }
   }
 }
 
  
-void Level::candidateFixed(SplitNode *splitNode,
+void Level::candidateFixed(SplitFrontier *splitFrontier,
                            unsigned int splitIdx,
                            const double ruPred[],
                            BHPair heap[],
-                           unsigned int extent,
                            unsigned int &spanCand) {
   // Inserts negative, weighted probability value:  choose from lowest.
   for (unsigned int predIdx = 0; predIdx < nPred; predIdx++) {
@@ -254,34 +275,31 @@ void Level::candidateFixed(SplitNode *splitNode,
   unsigned int schedCount = 0;
   for (unsigned int heapSize = nPred; heapSize > 0; heapSize--) {
     unsigned int predIdx = BHeap::slotPop(heap, heapSize - 1);
-    schedCount += preschedule(splitNode, splitIdx, predIdx, extent, spanCand) ? 1 : 0;
+    schedCount += preschedule(splitFrontier, SplitCoord(splitIdx, predIdx), spanCand) ? 1 : 0;
     if (schedCount == predFixed)
       break;
   }
 }
 
 
-bool Level::preschedule(SplitNode *splitNode,
-                        unsigned int splitIdx,
-                        unsigned int predIdx,
-                        unsigned int extent,
+bool Level::preschedule(SplitFrontier *splitFrontier,
+                        const SplitCoord& splitCoord,
                         unsigned int &spanCand) {
-  bottom->reachFlush(splitIdx, predIdx);
+  bottom->reachFlush(splitCoord.nodeIdx, splitCoord.predIdx);
 
   unsigned int bufIdx;
-  if (!isSingleton(splitIdx, predIdx, bufIdx)) {
-    splitNode->preschedule(splitIdx, predIdx, bufIdx);
-    offCand[pairOffset(splitIdx, predIdx)] = spanCand;
-    spanCand += extent;
+  if (!isSingleton(splitCoord, bufIdx)) {
+    offCand[splitCoord.strideOffset(nPred)] = spanCand;
+    spanCand += splitFrontier->preschedule(splitCoord, bufIdx);
     return true;
   }
   return false;
 }
 
 
-void Level::rankRestage(SamplePred *samplePred,
-                        const SPPair &mrra,
-                        Level *levelFront,
+void Level::rankRestage(ObsPart* samplePred,
+                        const SplitCoord& mrra,
+                        Level* levelFront,
                         unsigned int bufIdx) {
   unsigned int reachOffset[NodePath::pathMax()];
   if (nodeRel) { // Both levels employ node-relative indexing.
@@ -296,65 +314,63 @@ void Level::rankRestage(SamplePred *samplePred,
 }
 
 
-void Level::rankRestage(SamplePred *samplePred,
-                        const SPPair &mrra,
-                        Level *levelFront,
+void Level::rankRestage(ObsPart* samplePred,
+                        const SplitCoord& mrra,
+                        Level* levelFront,
                         unsigned int bufIdx,
                         unsigned int reachOffset[],
                         const unsigned int reachBase[]) {
-  unsigned int startIdx, extent;
-  getBounds(mrra, startIdx, extent);
-
+  IndexRange idxRange = getRange(mrra);
   unsigned int pathCount[NodePath::pathMax()];
   fill(pathCount, pathCount + backScale(1), 0);
 
-  unsigned int predIdx = mrra.second;
-  samplePred->prepath(nodeRel ?  getFrontPath() : bottom->subtreePath(), reachBase, predIdx, bufIdx, startIdx, extent, pathMask(), reachBase == nullptr ? levelFront->isNodeRel() : true, pathCount);
+  unsigned int predIdx = mrra.predIdx;
+  samplePred->prepath(nodeRel ?  getFrontPath() : bottom->getSubtreePath(), reachBase, predIdx, bufIdx, idxRange, pathMask(), reachBase == nullptr ? levelFront->isNodeRel() : true, pathCount);
 
   // Successors may or may not themselves be dense.
-  packDense(startIdx, pathCount, levelFront, mrra, reachOffset);
+  packDense(idxRange.getStart(), pathCount, levelFront, mrra, reachOffset);
 
   unsigned int rankPrev[NodePath::pathMax()];
   unsigned int rankCount[NodePath::pathMax()];
   fill(rankPrev, rankPrev + backScale(1), bottom->getNoRank());
   fill(rankCount, rankCount + backScale(1), 0);
 
-  samplePred->rankRestage(predIdx, bufIdx, startIdx, extent, reachOffset, rankPrev, rankCount);
+  samplePred->rankRestage(predIdx, bufIdx, idxRange, reachOffset, rankPrev, rankCount);
   setRunCounts(bottom, mrra, pathCount, rankCount);
 }
 
 
-void Level::packDense(unsigned int idxLeft,
+void Level::packDense(IndexType idxStart,
                       const unsigned int pathCount[],
-                      Level *levelFront,
-                      const SPPair &mrra,
+                      Level* levelFront,
+                      const SplitCoord& mrra,
                       unsigned int reachOffset[]) const {
-  if (!isDense(mrra.first, mrra.second)) {
+  if (!isDense(mrra)) {
     return;
   }
-  const NodePath *pathPos = &nodePath[backScale(mrra.first)];
+  const NodePath* pathPos = &nodePath[mrra.backScale(del)];
   for (unsigned int path = 0; path < backScale(1); path++) {
-    unsigned int levelIdx, idxStart, extent;
-    pathPos[path].getCoords(levelIdx, idxStart, extent);
-    if (levelIdx != noIndex) {
-      unsigned int margin = idxStart - idxLeft;
-      unsigned int extentDense = pathCount[path];
-      levelFront->setDense(levelIdx, mrra.second, extent - extentDense, margin);
+    IndexRange idxRange;
+    IndexType splitIdx = pathPos[path].getCoords(idxRange);
+    if (splitIdx != noIndex) {
+      IndexType margin = idxRange.getStart() - idxStart;
+      IndexType extentDense = pathCount[path];
+      levelFront->setDense(SplitCoord(splitIdx, mrra.predIdx), idxRange.getExtent() - extentDense, margin);
       reachOffset[path] -= margin;
-      idxLeft += extentDense;
+      idxStart += extentDense;
     }
   }
 }
 
 
-void Level::setRunCounts(Bottom *bottom, const SPPair &mrra, const unsigned int pathCount[], const unsigned int rankCount[]) const {
-  unsigned int predIdx = mrra.second;
-  const NodePath *pathPos = &nodePath[backScale(mrra.first)];
+void Level::setRunCounts(Bottom *bottom, const SplitCoord &mrra, const unsigned int pathCount[], const unsigned int rankCount[]) const {
+  unsigned int predIdx = mrra.predIdx;
+  const NodePath* pathPos = &nodePath[mrra.backScale(del)];
   for (unsigned int path = 0; path < backScale(1); path++) {
-    unsigned int levelIdx, idxStart, extent;
-    pathPos[path].getCoords(levelIdx, idxStart, extent);
-    if (levelIdx != noIndex) {
-      bottom->setRunCount(levelIdx, predIdx, pathCount[path] != extent, rankCount[path]);
+    IndexRange idxRange;
+    IndexType splitIdx = pathPos[path].getCoords(idxRange);
+    if (splitIdx != noIndex) {
+      bottom->setRunCount(SplitCoord(splitIdx, predIdx), pathCount[path] != idxRange.getExtent(), rankCount[path]);
     }
   }
 }
@@ -368,25 +384,25 @@ void Level::setRunCounts(Bottom *bottom, const SPPair &mrra, const unsigned int
 
    @param reachOffset holds the starting offset positions along the path.
  */
-void Level::offsetClone(const SPPair &mrra,
+void Level::offsetClone(const SplitCoord& mrra,
                         unsigned int reachOffset[],
                         unsigned int splitOffset[],
                         unsigned int reachBase[]) {
-  unsigned int nodeStart = backScale(mrra.first);
+  unsigned int nodeStart = mrra.backScale(del);
   for (unsigned int i = 0; i < backScale(1); i++) {
-    reachOffset[i] = nodePath[nodeStart + i].IdxStart();
-    splitOffset[i] = offCand[pairOffset(mrra.first, mrra.second)];
+    reachOffset[i] = nodePath[nodeStart + i].getIdxStart();
+    splitOffset[i] = offCand[mrra.strideOffset(nPred)];
   }
   if (reachBase != nullptr) {
     for (unsigned int i = 0; i < backScale(1); i++) {
-      reachBase[i] = nodePath[nodeStart + i].RelBase();
+      reachBase[i] = nodePath[nodeStart + i].getRelBase();
     }
   }
 }
 
 
-void Level::indexRestage(SamplePred *samplePred,
-                         const SPPair &mrra,
+void Level::indexRestage(ObsPart *samplePred,
+                         const SplitCoord &mrra,
                          const Level *levelFront,
                          unsigned int bufIdx) {
   unsigned int reachOffset[NodePath::pathMax()];
@@ -403,20 +419,17 @@ void Level::indexRestage(SamplePred *samplePred,
 }
 
 
-void Level::indexRestage(SamplePred *samplePred,
-                         const SPPair &mrra,
+void Level::indexRestage(ObsPart* obsPart,
+                         const SplitCoord& mrra,
                          const Level *levelFront,
                          unsigned int bufIdx,
                          const unsigned int reachBase[],
                          unsigned int reachOffset[],
                          unsigned int splitOffset[]) {
-  unsigned int startIdx, extent;
-  getBounds(mrra, startIdx, extent);
-
-  samplePred->indexRestage(nodeRel ? getFrontPath() : bottom->subtreePath(),
-                           reachBase, mrra.second, bufIdx, startIdx, extent,
-                           pathMask(),
-                           reachBase == nullptr ? levelFront->isNodeRel() : true,
-                           reachOffset,
-                           splitOffset);
+  obsPart->indexRestage(nodeRel ? getFrontPath() : bottom->getSubtreePath(),
+                        reachBase, mrra, bufIdx, getRange(mrra), 
+                        pathMask(),
+                        reachBase == nullptr ? levelFront->isNodeRel() : true,
+                        reachOffset,
+                        splitOffset);
 }
