@@ -14,72 +14,14 @@
 
  */
 
-#ifndef ARBORIST_PRETREE_H
-#define ARBORIST_PRETREE_H
+#ifndef PARTITION_PRETREE_H
+#define PARTITION_PRETREE_H
 
 #include <vector>
 #include <algorithm>
 
-#include "decnode.h"
-
-
-/**
-  @brief DecNode specialized for training.
- */
-class PTNode : public DecNode {
-  FltVal info;  // Nonzero iff nonterminal.
- public:
-  
-  void consumeNonterminal(const class FrameTrain *frameTrain,
-                          class ForestTrain *forest,
-                          vector<double> &predInfo,
-                          unsigned int idx) const;
-
-  void splitNum(const class SplitCand &cand,
-                unsigned int lhDel);
-
-  /**
-     @brief Resets to default terminal status.
-
-     @return void.
-   */
-  inline void setTerminal() {
-    lhDel = 0;
-  }
-
-
-  /**
-     @brief Resets to nonterminal with specified lh-delta.
-
-     @return void.
-   */
-  inline void setNonterminal(unsigned int lhDel) {
-    this->lhDel = lhDel;
-  }
-
-  
-  inline bool isNonTerminal() const {
-    return lhDel != 0;
-  }
-
-
-  inline unsigned int getLHId(unsigned int ptId) const {
-    return isNonTerminal() ? ptId + lhDel : 0;
-  }
-
-  inline unsigned int getRHId(unsigned int ptId) const {
-    return isNonTerminal() ? getLHId(ptId) + 1 : 0;
-  }
-
-
-  inline void SplitFac(unsigned int predIdx, unsigned int lhDel, unsigned int bitEnd, double info) {
-    this->predIdx = predIdx;
-    this->lhDel = lhDel;
-    this->splitVal.offset = bitEnd;
-    this->info = info;
-  }
-};
-
+#include "ptnode.h" // Algorithm-dependent definition.
+#include "decnode.h" 
 
 /**
  @brief Serialized representation of the pre-tree, suitable for tranfer between
@@ -88,18 +30,29 @@ class PTNode : public DecNode {
 class PreTree {
   static size_t heightEst;
   static size_t leafMax; // User option:  maximum # leaves, if > 0.
-  const class FrameTrain *frameTrain;
   const unsigned int bagCount;
-  size_t nodeCount; // Allocation height of node vector.
-  PTNode *nodeVec; // Vector of tree nodes.
   size_t height;
   size_t leafCount;
   size_t bitEnd; // Next free slot in factor bit vector.
+  vector<class PTNode> nodeVec; // Vector of tree nodes.
+  vector<struct SplitCrit> splitCrit;
   class BV *splitBits;
   vector<unsigned int> termST;
-  class BV *BitFactory();
+
+  /**
+     @brief Constructs mapping from sample indices to leaf indices.
+
+     @param[in, out] forest accumulates the growing forest.
+
+     @return rewritten map.
+  */
   const vector<unsigned int> frontierConsume(class ForestTrain *forest) const;
-  unsigned int BitWidth();
+
+
+  /**
+     @return BV-aligned length of used portion of split vector.
+  */
+  IndexT getBitWidth();
 
 
   /**
@@ -107,7 +60,7 @@ class PreTree {
 
      @return void, with incremented height and leaf count.
   */
-  inline void TerminalOffspring() {
+  inline void terminalOffspring() {
   // Two more leaves for offspring, one fewer for this.
     height += 2;
     leafCount++;
@@ -115,13 +68,51 @@ class PreTree {
 
 
  public:
-  PreTree(const class FrameTrain *_frameTrain, unsigned int _bagCount);
+  PreTree(const class SummaryFrame* frame,
+          const class Frontier* frontier);
+
   ~PreTree();
   static void immutables(size_t _nSamp, size_t _minH, size_t _leafMax);
   static void deImmutables();
+
+
+  /**
+     @brief Refines the height estimate using the actual height of a
+     constructed PreTree.
+
+     @param height is an actual height value.
+  */
   static void reserve(size_t height);
 
 
+  /**
+     @brief Dispatches nonterminal method according to split type.
+   */
+  void nonterminal(double info,
+                   class IndexSet* iSet);
+
+  
+  /**
+     @brief Appends criterion for bit-based branch.
+
+     @param predIdx is the criterion predictor.
+
+     @param cardinality is the predictor's cardinality.
+  */
+  void critBits(const class IndexSet* iSet,
+                unsigned int predIdx,
+                unsigned int cardinality);
+
+  /**
+     @brief Appends criterion for cut-based branch.
+     
+     @param rankRange bounds the cut-defining ranks.
+  */
+  void critCut(const class IndexSet* iSet,
+               unsigned int predIdx,
+               const IndexRange& rankRange);
+
+
   /**
      @brief Consumes all pretree nonterminal information into crescent forest.
 
@@ -136,46 +127,64 @@ class PreTree {
   const vector<unsigned int> consume(class ForestTrain *forest,
                                      unsigned int tIdx,
                                      vector<double> &predInfo);
+  /**
+     @brief Consumes nonterminal information into the dual-use vectors needed by the decision tree.
 
+     Leaf information is post-assigned by the response-dependent Sample methods.
+
+     @param[in, out]  forest inputs/outputs the updated forest.
+
+     @param[out] predInfo outputs the predictor-specific information values.
+  */
   void consumeNonterminal(class ForestTrain *forest,
                           vector<double> &predInfo) const;
 
   void BitConsume(unsigned int *outBits);
 
-  void LHBit(int idx, unsigned int pos);
-
-  void branchFac(const class SplitCand& argMax,
-                 unsigned int _id);
 
   /**
-     @brief Finalizes numeric-valued nonterminal.
+     @brief Sets specified bit in (left) splitting bit vector.
 
-     @param argMax is the split candidate characterizing the branch.
+     @param iSet is the index node for which the LH bit is set.
 
-     @param id is the node index.
+     @param pos is the bit position beyond to set.
   */
-  void branchNum(const class SplitCand &argMax,
-                 unsigned int id);
+  void setLeft(const class IndexSet* iSet,
+               IndexT pos);
 
-  void levelStorage(unsigned int splitNext, unsigned int leafNext);
-  void ReNodes();
+
+  /**
+     @brief Absorbs the terminal list from a completed subtree.
+
+     Side-effects the frontier map.
+
+     @param stTerm are subtree-relative indices.  These must be mapped to
+     sample indices if the subtree is proper.
+  */
   void subtreeFrontier(const vector<unsigned int> &stTerm);
-  unsigned int LeafMerge();
+
   
-  inline unsigned int getLHId(unsigned int ptId) const {
+  unsigned int LeafMerge();
+
+  
+  inline IndexT getLHId(IndexT ptId) const {
     return nodeVec[ptId].getLHId(ptId);
   }
 
   
-  inline unsigned int getRHId(unsigned int ptId) const {
+  inline IndexT getRHId(IndexT ptId) const {
     return nodeVec[ptId].getRHId(ptId);
   }
 
+
+  inline IndexT getSuccId(IndexT ptId, bool isLeft) const {
+    return isLeft ? nodeVec[ptId].getLHId(ptId) : nodeVec[ptId].getRHId(ptId);
+  }
   
   /**
      @return true iff node is nonterminal.
    */
-  inline bool isNonTerminal(unsigned int ptId) const {
+  inline bool isNonTerminal(IndexT ptId) const {
     return nodeVec[ptId].isNonTerminal();
   }
 
@@ -188,7 +197,7 @@ class PreTree {
 
        @return true iff node has two leaf children.
     */
-  inline bool isMergeable(unsigned int ptId) const {
+  inline bool isMergeable(IndexT ptId) const {
     return !isNonTerminal(getLHId(ptId)) && !isNonTerminal(getRHId(ptId));
   }  
 
@@ -206,7 +215,7 @@ class PreTree {
                         size_t &_bagCount) {
     _height += height;
     _maxHeight = max(height, _maxHeight);
-    _bitWidth += BitWidth();
+    _bitWidth += getBitWidth();
     _leafCount += leafCount;
     _bagCount += bagCount;
   }
